from abaqusConstants import *


class MovieOptions:
    """The MovieOptions object stores settings that control how the movie background for an
    animation is rendered in a particular viewport. MovieOptions objects are accessed from
    the movie options associated with a particular viewport.
    The MovieOptions object has no constructor; Abaqus creates the **movieOptions** member for
    a viewport when the viewport is created using the values in the **movieOptions** member of
    the current viewport.

    Notes
    -----
    This object can be accessed by:

    .. code-block::

        session.viewports[name].movieOptions
    """

    def setValues(
        self,
        movieName: str = "",
        showMovie: Boolean = OFF,
        positionMethod: SymbolicConstant = FIT_TO_VIEWPORT,
        fitMethod: SymbolicConstant = BEST_FIT,
        alignment: SymbolicConstant = CENTER,
        xScale: float = 1,
        yScale: float = 1,
        origin: tuple[float, ...] = (),
        translucency: float = 1,
        options: str = None,
    ):
        """This method modifies the MovieOptions object.

        Parameters
        ----------
        movieName
            A String specifying the name of the movie. A list of valid movie names is in the
            **movies** repository in the **session** object.
        showMovie
            A Boolean specifying whether a movie should be displayed in the viewport during an
            animation. The default value is OFF.
        positionMethod
            A SymbolicConstant specifying which positioning method is used to determine how the
            movie frames will be scaled and positioned in the viewport. Possible values
            are:FIT_TO_VIEWPORT, specifying a display mode where the movie frame is scaled to fit in
            the viewport using the specified **fitMethod**.AUTO_ALIGN, specifying a display mode where
            the movie frame is scaled as specified by **xScale** and **yScale** and then positioned in
            the viewport using the specified **alignment**.MANUAL, specifying a display mode where the
            movie frame is scaled as specified by **xScale** and **yScale** and then positioned in the
            viewport using the specified **origin**.The default value is FIT_TO_VIEWPORT.
        fitMethod
            A SymbolicConstant specifying which type of fit is performed to scale and position the
            movie frame in the viewport when **positionMethod** =FIT_TO_VIEWPORT. Possible values
            are:BEST_FIT, specifying a mode where the movie frame is scaled to completely fit within
            the viewport.FIT_WIDTH, specifying a mode where the movie frame width is scaled to match
            the viewport width.FIT_HEIGHT, specifying a mode where the movie frame height is scaled
            to match the viewport height.The default value is BEST_FIT.
        alignment
            A SymbolicConstant specifying the relative position of the movie frame in the viewport
            when **positionMethod** =AUTO_ALIGN. Possible values
            are:BOTTOM_LEFTBOTTOM_CENTERBOTTOM_RIGHTCENTER_LEFTCENTERCENTER_RIGHTTOP_LEFTTOP_CENTERTOP_RIGHTThe
            default value is CENTER.
        xScale
            A Float specifying the scale applied to the movie frame width. The **xScale** argument is
            ignored when **positionMethod** =FIT_TO_VIEWPORT. The default value is 1.0.When **xScale**
            is negative, the movie frame is mirrored about its y-axis but its position is not
            affected.
        yScale
            A Float specifying the scale applied to the movie frame height. The **yScale** argument is
            ignored when **positionMethod** =FIT_TO_VIEWPORT. The default value is 1.0.When **yScale**
            is negative, the movie frame is mirrored about its x-axis but its position is not
            affected.
        origin
            A pair of Floats specifying the*X*- and **Y**-offsets in millimeters from the lower-left
            corner of the viewport. The **origin** argument is ignored unless **positionMethod**
            =MANUAL. The default value is (0, 0).
        translucency
            A Float specifying the translucency factor to use when displaying the movie frame.
            Possible values are 0.0 ≤≤ **translucency** ≤≤ 1.0 with 0.0 being invisible and 1.0 being
            opaque. The default value is 1.0.
        options
            None or a MovieOptions object specifying the object from which values are to be copied.
            If other arguments are also supplied to setValues, they will override the values in the
            **options** member. The default value is None.

        Raises
        ------
        RangeError
        """
        pass



from abaqusConstants import *
from .LoadState import LoadState


class ConcentratedForceState(LoadState):
    """The ConcentratedForceState object stores the propagating data for a concentrated force
    in a step. One instance of this object is created internally by the ConcentratedForce
    object for each step. The instance is also deleted internally by the ConcentratedForce
    object.
    The ConcentratedForceState object has no constructor or methods.
    The ConcentratedForceState object is derived from the LoadState object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - CLOAD
    """

    #: A Float or a Complex specifying the concentrated force component in the 1-direction.
    #: Although **cf1**, **cf2**, and **cf3** are optional arguments, at least one of them must be
    #: nonzero.
    cf1: float = None

    #: A Float or a Complex specifying the concentrated force component in the 2-direction.
    cf2: float = None

    #: A Float or a Complex specifying the concentrated force component in the 3-direction.
    cf3: float = None

    #: A SymbolicConstant specifying the propagation state of the concentrated force component
    #: in the 1-direction. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    cf1State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the concentrated force component
    #: in the 2-direction. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    cf2State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the concentrated force component
    #: in the 3-direction. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    cf3State: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: SymbolicConstant = None

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    status: SymbolicConstant = None

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    amplitude: str = ""



from abaqusConstants import *
from .Interaction import Interaction
from ..Region.Region import Region


class ModelChange(Interaction):
    """The ModelChange object defines model change interactions for element removal and
    reactivation.
    The ModelChange object is derived from the Interaction object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import interaction
            mdb.models[name].interactions[name]

        The corresponding analysis keywords are:

        - MODEL CHANGE
    """

    #: A String specifying the repository key.
    name: str

    #: A String specifying the name of the step in which the ModelChange object is created.
    createStepName: str

    #: A Boolean specifying whether this interaction is being used solely to indicate that
    #: model change may be required in a subsequent restart analysis (either for elements or
    #: contact pairs). The default value is OFF.
    isRestart: Boolean = OFF

    #: A SymbolicConstant specifying the region selection type. This argument is valid only
    #: when **isRestart** = False. Possible values are GEOMETRY, SKINS, STRINGERS, and ELEMENTS.
    #: The default value is GEOMETRY.
    regionType: SymbolicConstant = GEOMETRY

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the elements to be removed or reactivated. This argument is
    #: valid only when **isRestart** = False.
    region: Region = None

    #: A Boolean specifying whether elements are being removed or reactivated. This argument is
    #: valid only when **isRestart** = False. The default value is OFF.
    activeInStep: Boolean = OFF

    #: A Boolean specifying whether stress/displacement elements are reactivated with strain.
    #: This argument is valid only when **isRestart** = False and when **activeInStep** = True. The
    #: default value is OFF.
    includeStrain: Boolean = OFF

    def __init__(
        self,
        name: str,
        createStepName: str,
        isRestart: Boolean = OFF,
        regionType: SymbolicConstant = GEOMETRY,
        region: Region = None,
        activeInStep: Boolean = OFF,
        includeStrain: Boolean = OFF,
    ):
        """This method creates a ModelChange object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].ModelChange

        Parameters
        ----------
        name
            A String specifying the repository key.
        createStepName
            A String specifying the name of the step in which the ModelChange object is created.
        isRestart
            A Boolean specifying whether this interaction is being used solely to indicate that
            model change may be required in a subsequent restart analysis (either for elements or
            contact pairs). The default value is OFF.
        regionType
            A SymbolicConstant specifying the region selection type. This argument is valid only
            when **isRestart** = False. Possible values are GEOMETRY, SKINS, STRINGERS, and ELEMENTS.
            The default value is GEOMETRY.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the elements to be removed or reactivated. This argument is
            valid only when **isRestart** = False.
        activeInStep
            A Boolean specifying whether elements are being removed or reactivated. This argument is
            valid only when **isRestart** = False. The default value is OFF.
        includeStrain
            A Boolean specifying whether stress/displacement elements are reactivated with strain.
            This argument is valid only when **isRestart** = False and when **activeInStep** = True. The
            default value is OFF.

        Returns
        -------
        ModelChange
            A :py:class:`~abaqus.Interaction.ModelChange.ModelChange` object.
        """
        super().__init__()
        pass

    def setValues(
        self,
        isRestart: Boolean = OFF,
        regionType: SymbolicConstant = GEOMETRY,
        region: Region = None,
        activeInStep: Boolean = OFF,
        includeStrain: Boolean = OFF,
    ):
        """This method modifies the data for an existing ModelChange object in the step where it is
        created.

        Parameters
        ----------
        isRestart
            A Boolean specifying whether this interaction is being used solely to indicate that
            model change may be required in a subsequent restart analysis (either for elements or
            contact pairs). The default value is OFF.
        regionType
            A SymbolicConstant specifying the region selection type. This argument is valid only
            when **isRestart** = False. Possible values are GEOMETRY, SKINS, STRINGERS, and ELEMENTS.
            The default value is GEOMETRY.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the elements to be removed or reactivated. This argument is
            valid only when **isRestart** = False.
        activeInStep
            A Boolean specifying whether elements are being removed or reactivated. This argument is
            valid only when **isRestart** = False. The default value is OFF.
        includeStrain
            A Boolean specifying whether stress/displacement elements are reactivated with strain.
            This argument is valid only when **isRestart** = False and when **activeInStep** = True. The
            default value is OFF.
        """
        pass

    def setValuesInStep(
        self, stepName: str, activeInStep: Boolean = OFF, includeStrain: Boolean = OFF
    ):
        """This method modifies the propagating data of an existing ModelChange object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the interaction is modified.
        activeInStep
            A Boolean specifying whether elements are being removed or reactivated. This argument is
            valid only when **isRestart** = False. The default value is OFF.
        includeStrain
            A Boolean specifying whether stress/displacement elements are reactivated with strain.
            This argument is valid only when **isRestart** = False and when **activeInStep** = True. The
            default value is OFF.
        """
        pass



from abaqusConstants import *
from .BoundaryCondition import BoundaryCondition
from ..Region.Region import Region


class SubmodelBC(BoundaryCondition):
    """The SubmodelBC object stores the data for a submodel boundary condition.
    The SubmodelBC object is derived from the BoundaryCondition object.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A Float specifying the thickness of the shell in the global model. This argument is
    #: required for shell-to-solid submodeling and is not applicable to other submodels. The
    #: default value is 0.0.
    shellThickness: float = 0

    #: None or a Float specifying the absolute value by which a driven node of the submodel can
    #: lie outside the region of the elements of the global model. The default value is None.
    absoluteExteriorTolerance: float = None

    #: None or a Float specifying the fraction of the average element size in the global model
    #: by which a driven node of the submodel can lie outside the region of the elements of the
    #: global model. The default value is 0.05.
    exteriorTolerance: float = 0

    #: A String specifying the element set in the global model that will be searched for
    #: elements whose responses will be used to drive the submodel. An empty string indicates
    #: that the entire global model will be searched. The default value is an empty string.
    globalDrivingRegion: str = ""

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: SymbolicConstant = None

    #: A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: str = None

    def __init__(
        self,
        name: str,
        createStepName: str,
        region: Region,
        dof: tuple,
        globalStep: str,
        timeScale: Boolean,
        shellThickness: float,
        globalDrivingRegion: str = "",
        absoluteExteriorTolerance: float = None,
        exteriorTolerance: float = 0,
        localCsys: str = None,
        globalIncrement: int = 0,
        centerZoneSize: float = None,
    ):
        """This method creates a SubmodelBC object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].SubmodelBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A :py:class:`~abaqus.Region.Region.Region` object specifying the region to which the boundary condition is applied.
        dof
            A sequence of Ints specifying the degrees of freedom to which the boundary condition is
            applied.
        globalStep
            A String specifying the step in the global model from which Abaqus reads the values of
            the variables that will drive the submodel analysis. The String indicates the position
            of the step in the sequence of analysis steps. For example, **globalStep** = '1' indicates
            the first step.
        timeScale
            A Boolean specifying whether to scale the time variable for the driven nodes' amplitude
            functions to match the submodel analysis step time. The default value is OFF.
        shellThickness
            A Float specifying the thickness of the shell in the global model. This argument is
            required for shell-to-solid submodeling and is not applicable to other submodels. The
            default value is 0.0.
        globalDrivingRegion
            A String specifying the element set in the global model that will be searched for
            elements whose responses will be used to drive the submodel. An empty string indicates
            that the entire global model will be searched. The default value is an empty string.
        absoluteExteriorTolerance
            None or a Float specifying the absolute value by which a driven node of the submodel can
            lie outside the region of the elements of the global model. The default value is None.
        exteriorTolerance
            None or a Float specifying the fraction of the average element size in the global model
            by which a driven node of the submodel can lie outside the region of the elements of the
            global model. The default value is 0.05.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        globalIncrement
            An Int specifying the increment number in the global model step from which the solution
            will be used to specify the values of the driven variables. If **globalIncrement** = 0, the
            solution from the last increment will be used. The **globalIncrement** argument is
            applicable only for linear perturbation steps. The default value is 0.
        centerZoneSize
            A Float specifying the thickness of the center zone size around the shell midsurface.
            The default value is None.

        Returns
        -------
        SubmodelBC
            A :py:class:`~abaqus.BoundaryCondition.SubmodelBC.SubmodelBC` object.
        """
        super().__init__()
        pass

    def setValues(
        self,
        globalDrivingRegion: str = "",
        absoluteExteriorTolerance: float = None,
        exteriorTolerance: float = 0,
        localCsys: str = None,
        globalIncrement: int = 0,
        centerZoneSize: float = None,
    ):
        """This method modifies the data for an existing SubmodelBC object in the step where it is
        created.

        Parameters
        ----------
        globalDrivingRegion
            A String specifying the element set in the global model that will be searched for
            elements whose responses will be used to drive the submodel. An empty string indicates
            that the entire global model will be searched. The default value is an empty string.
        absoluteExteriorTolerance
            None or a Float specifying the absolute value by which a driven node of the submodel can
            lie outside the region of the elements of the global model. The default value is None.
        exteriorTolerance
            None or a Float specifying the fraction of the average element size in the global model
            by which a driven node of the submodel can lie outside the region of the elements of the
            global model. The default value is 0.05.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        globalIncrement
            An Int specifying the increment number in the global model step from which the solution
            will be used to specify the values of the driven variables. If **globalIncrement** = 0, the
            solution from the last increment will be used. The **globalIncrement** argument is
            applicable only for linear perturbation steps. The default value is 0.
        centerZoneSize
            A Float specifying the thickness of the center zone size around the shell midsurface.
            The default value is None.
        """
        pass

    def setValuesInStep(
        self,
        stepName: str,
        fixed: Boolean = ON,
        dof: tuple = (),
        globalStep: str = "",
        globalIncrement: int = 0,
        centerZoneSize: float = None,
    ):
        """This method modifies the propagating data for an existing SubmodelBC object in the
        specified step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is ON.
        dof
            A sequence of Ints specifying the degrees of freedom to which the boundary condition is
            applied. The **dof** argument is applicable only if **fixed** = OFF.
        globalStep
            A String specifying the step in the global model from which Abaqus reads the values of
            the variables that will drive the submodel analysis. The String indicates the position
            of the step in the sequence of analysis steps. For example, **globalStep** = '1' indicates
            the first step. The **globalStep** argument is applicable only if **fixed** = OFF.
        globalIncrement
            An Int specifying the increment number in the global model step at which the solution
            will be used to specify the values of the driven variables. If **globalIncrement** = 0, the
            solution from the last increment will be used. The **globalIncrement** argument is
            applicable only for linear perturbation steps and if **fixed** = OFF. The default value is
            0.
        centerZoneSize
            A Float specifying the thickness of the center zone size around the shell midsurface.
            The default value is None.The **centerZoneSize** argument is applicable only if
            **fixed** = OFF.
        """
        pass



from abaqus.Odb.OdbCommands import *
from abaqus.UtilityAndView.BackwardCompatibility import BackwardCompatibility


def openOdb(name: str, *args, **kwargs):
    abaqus = 'abaqus'
    if 'ABAQUS_BAT_PATH' in os.environ.keys():
        abaqus = os.environ['ABAQUS_BAT_PATH']

    filePath = os.path.abspath(sys.argv[0])
    fileDir = os.path.dirname(filePath)
    fileName = os.path.basename(filePath)
    odbName = os.path.basename(os.path.abspath(name))

    os.system(f'cd {fileDir}')
    try:  # If it is a jupyter notebook
        import ipynbname
        fileName = os.path.basename(ipynbname.path())
        os.system(f'jupyter nbconvert --to python {fileName}')
        fileName = fileName.replace('.ipynb', '.py')
    except:
        pass
    os.system(f'{abaqus} cae database={odbName} script={fileName}')
    os.system('exit')
    return Odb(name)


backwardCompatibility = BackwardCompatibility()



class MeshStats:
    """The MeshStats object is a query object for holding mesh statistics and is returned by
    the getMeshStats command. The object does not have any methods.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import mesh
    """

    #: An Int specifying the number of point elements.
    numPointElems: int = None

    #: An Int specifying the number of line elements.
    numLineElems: int = None

    #: An Int specifying the number of quadrilateral elements.
    numQuadElems: int = None

    #: An Int specifying the number of triangular elements.
    numTriElems: int = None

    #: An Int specifying the number of hexahedral elements.
    numHexElems: int = None

    #: An Int specifying the number of wedge elements.
    numWedgeElems: int = None

    #: An Int specifying the number of tetrahedral elements.
    numTetElems: int = None

    #: An Int specifying the number of pyramid elements.
    numPyramidElems: int = None

    #: An Int specifying the number of nodes.
    numNodes: int = None

    #: An Int specifying the number of regions that contain a mesh.
    numMeshedRegions: int = None



class Radiation:
    """The Radiation object specifies radiation for a contact interaction property.

    .. note:: 
        This object can be accessed by:

        .. code-block:: python

            import interaction
            mdb.models[name].interactionProperties[name].radiation

        The corresponding analysis keywords are:

        - GAP RADIATION
    """

    #: A Float specifying the emissivity of the main surface.
    mainEmissivity: float

    #: A Float specifying the emissivity of the secondary surface.
    secondaryEmissivity: float

    #: A sequence of sequences of Floats specifying the following:Effective viewfactor, FF.Gap
    #: clearance, dd.
    table: tuple

    def __init__(self, mainEmissivity: float, secondaryEmissivity: float, table: tuple):
        """This method creates a Radiation object.

        .. note:: 
            This function can be accessed by:

            .. code-block:: python

                mdb.models[name].interactionProperties[name].Radiation

        Parameters
        ----------
        mainEmissivity
            A Float specifying the emissivity of the main surface.
        secondaryEmissivity
            A Float specifying the emissivity of the secondary surface.
        table
            A sequence of sequences of Floats specifying the following:Effective viewfactor, FF.Gap
            clearance, dd.

        Returns
        -------
        Radiation
            A :py:class:`~abaqus.Interaction.Radiation.Radiation` object.
        """
        pass

    def setValues(self, *args, **kwargs):
        """This method modifies the Radiation object."""
        pass
